@@ -230,7 +230,7 @@ ULONG WINAPI EnableTraceEx( LPCGUID provider, LPCGUID source, TRACEHANDLE hSessi
                             UCHAR level, ULONGLONG anykeyword, ULONGLONG allkeyword, ULONG enableprop,
                             PEVENT_FILTER_DESCRIPTOR filterdesc )
 {
-    FIXME("(%s, %s, %s, %d, %c, %s, %s, %d, %p): stub\n", debugstr_guid(provider),
+    FIXME("(%s, %s, %s, %u, %u, %s, %s, %u, %p): stub\n", debugstr_guid(provider),
             debugstr_guid(source), wine_dbgstr_longlong(hSession), enable, level,
             wine_dbgstr_longlong(anykeyword), wine_dbgstr_longlong(allkeyword),
             enableprop, filterdesc);
@@ -238,6 +238,20 @@ ULONG WINAPI EnableTraceEx( LPCGUID provider, LPCGUID source, TRACEHANDLE hSessi
     return ERROR_SUCCESS;
 }
 
+/******************************************************************************
+ * EnableTraceEx2 [ADVAPI32.@]
+ */
+ULONG WINAPI EnableTraceEx2( TRACEHANDLE handle, LPCGUID provider, ULONG control, UCHAR level,
+                             ULONGLONG match_any, ULONGLONG match_all, ULONG timeout,
+                             PENABLE_TRACE_PARAMETERS params )
+{
+    FIXME("(%s, %s, %u, %u, %s, %s, %u, %p): stub\n", wine_dbgstr_longlong(handle),
+          debugstr_guid(provider), control, level, wine_dbgstr_longlong(match_any),
+          wine_dbgstr_longlong(match_all), timeout, params);
+
+    return ERROR_SUCCESS;
+}
+
 /******************************************************************************
  * EnableTrace [ADVAPI32.@]
  */
@@ -814,26 +828,6 @@ ULONG WINAPI UnregisterTraceGuids( TRACEHANDLE RegistrationHandle )
     return ERROR_CALL_NOT_IMPLEMENTED;
 }
 
-/******************************************************************************
- * EventRegister [ADVAPI32.@]
- */
-ULONG WINAPI EventRegister( LPCGUID provider, PENABLECALLBACK callback, PVOID context, PREGHANDLE handle )
-{
-    FIXME("%s, %p, %p, %p\n", debugstr_guid(provider), callback, context, handle);
-
-    *handle = 0xdeadbeef;
-    return ERROR_SUCCESS;
-}
-
-/******************************************************************************
- * EventUnregister [ADVAPI32.@]
- */
-ULONG WINAPI EventUnregister( REGHANDLE handle )
-{
-    FIXME("%s: stub\n", wine_dbgstr_longlong(handle));
-    return ERROR_SUCCESS;
-}
-
 /******************************************************************************
  * EventEnabled [ADVAPI32.@]
  *
