@@ -28,7 +28,7 @@
 #ifdef D2D1_INIT_GUID
 #include "initguid.h"
 #endif
-#include "dwrite.h"
+#include "dwrite_2.h"
 
 enum d2d_brush_type
 {
@@ -87,16 +87,14 @@ struct d2d_d3d_render_target
     IDWriteRenderingParams *text_rendering_params;
     IDWriteRenderingParams *default_text_rendering_params;
 
-    D2D1_PIXEL_FORMAT format;
+    D2D1_RENDER_TARGET_PROPERTIES desc;
     D2D1_SIZE_U pixel_size;
     struct d2d_clip_stack clip_stack;
-    float dpi_x;
-    float dpi_y;
 };
 
 HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target, ID2D1Factory *factory,
         IDXGISurface *surface, const D2D1_RENDER_TARGET_PROPERTIES *desc) DECLSPEC_HIDDEN;
-HRESULT d2d_d3d_render_target_update_surface(ID2D1RenderTarget *render_target, IDXGISurface1 *surface) DECLSPEC_HIDDEN;
+HRESULT d2d_d3d_render_target_create_rtv(ID2D1RenderTarget *render_target, IDXGISurface1 *surface) DECLSPEC_HIDDEN;
 
 struct d2d_wic_render_target
 {
@@ -131,6 +129,35 @@ struct d2d_dc_render_target
 HRESULT d2d_dc_render_target_init(struct d2d_dc_render_target *render_target, ID2D1Factory *factory,
         ID3D10Device1 *device, const D2D1_RENDER_TARGET_PROPERTIES *desc) DECLSPEC_HIDDEN;
 
+struct d2d_hwnd_render_target
+{
+    ID2D1HwndRenderTarget ID2D1HwndRenderTarget_iface;
+    LONG refcount;
+
+    ID2D1RenderTarget *dxgi_target;
+    IDXGISwapChain *swapchain;
+    UINT sync_interval;
+    HWND hwnd;
+};
+
+HRESULT d2d_hwnd_render_target_init(struct d2d_hwnd_render_target *render_target, ID2D1Factory *factory,
+        ID3D10Device1 *device, const D2D1_RENDER_TARGET_PROPERTIES *desc,
+        const D2D1_HWND_RENDER_TARGET_PROPERTIES *hwnd_desc) DECLSPEC_HIDDEN;
+
+struct d2d_bitmap_render_target
+{
+    ID2D1BitmapRenderTarget ID2D1BitmapRenderTarget_iface;
+    LONG refcount;
+
+    ID2D1RenderTarget *dxgi_target;
+    ID2D1Bitmap *bitmap;
+};
+
+HRESULT d2d_bitmap_render_target_init(struct d2d_bitmap_render_target *render_target,
+        const struct d2d_d3d_render_target *parent_target, const D2D1_SIZE_F *size,
+        const D2D1_SIZE_U *pixel_size, const D2D1_PIXEL_FORMAT *format,
+        D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS options) DECLSPEC_HIDDEN;
+
 struct d2d_gradient
 {
     ID2D1GradientStopCollection ID2D1GradientStopCollection_iface;
@@ -169,6 +196,11 @@ struct d2d_brush
             D2D1_BITMAP_INTERPOLATION_MODE interpolation_mode;
             ID3D10SamplerState *sampler_state;
         } bitmap;
+        struct
+        {
+            D2D1_LINEAR_GRADIENT_BRUSH_PROPERTIES desc;
+            ID2D1GradientStopCollection *gradient;
+        } linear;
     } u;
 };
 
@@ -221,7 +253,7 @@ struct d2d_bitmap
 
 HRESULT d2d_bitmap_create(ID2D1Factory *factory, ID3D10Device *device, D2D1_SIZE_U size, const void *src_data,
         UINT32 pitch, const D2D1_BITMAP_PROPERTIES *desc, struct d2d_bitmap **bitmap) DECLSPEC_HIDDEN;
-HRESULT d2d_bitmap_create_shared(ID2D1Factory *factory, ID3D10Device *device, REFIID iid, void *data,
+HRESULT d2d_bitmap_create_shared(ID2D1RenderTarget *render_target, ID3D10Device *device, REFIID iid, void *data,
         const D2D1_BITMAP_PROPERTIES *desc, struct d2d_bitmap **bitmap) DECLSPEC_HIDDEN;
 HRESULT d2d_bitmap_create_from_wic_bitmap(ID2D1Factory *factory, ID3D10Device *device, IWICBitmapSource *bitmap_source,
         const D2D1_BITMAP_PROPERTIES *desc, struct d2d_bitmap **bitmap) DECLSPEC_HIDDEN;
@@ -330,4 +362,27 @@ static inline void d2d_matrix_multiply(D2D_MATRIX_3X2_F *a, const D2D_MATRIX_3X2
     a->_32 = tmp._31 * b->_12 + tmp._32 * b->_22 + b->_32;
 }
 
+/* Dst must be different from src. */
+static inline BOOL d2d_matrix_invert(D2D_MATRIX_3X2_F *dst, const D2D_MATRIX_3X2_F *src)
+{
+    float d = src->_11 * src->_22 - src->_21 * src->_12;
+
+    if (d == 0.0f)
+        return FALSE;
+    dst->_11 = src->_22 / d;
+    dst->_21 = -src->_21 / d;
+    dst->_31 = (src->_21 * src->_32 - src->_31 * src->_22) / d;
+    dst->_12 = -src->_12 / d;
+    dst->_22 = src->_11 / d;
+    dst->_32 = -(src->_11 * src->_32 - src->_31 * src->_12) / d;
+
+    return TRUE;
+}
+
+static inline void d2d_point_transform(D2D1_POINT_2F *dst, const D2D1_MATRIX_3X2_F *matrix, float x, float y)
+{
+    dst->x = x * matrix->_11 + y * matrix->_21 + matrix->_31;
+    dst->y = x * matrix->_12 + y * matrix->_22 + matrix->_32;
+}
+
 #endif /* __WINE_D2D1_PRIVATE_H */
