@@ -48,12 +48,6 @@ static void d2d_point_set(D2D1_POINT_2F *dst, float x, float y)
     dst->y = y;
 }
 
-static void d2d_point_transform(D2D1_POINT_2F *dst, const D2D1_MATRIX_3X2_F *matrix, float x, float y)
-{
-    dst->x = x * matrix->_11 + y * matrix->_21 + matrix->_31;
-    dst->y = x * matrix->_12 + y * matrix->_22 + matrix->_32;
-}
-
 static void d2d_rect_expand(D2D1_RECT_F *dst, const D2D1_POINT_2F *point)
 {
     if (point->x < dst->left)
@@ -341,7 +335,7 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateSharedBitmap(ID2D1R
     TRACE("iface %p, iid %s, data %p, desc %p, bitmap %p.\n",
             iface, debugstr_guid(iid), data, desc, bitmap);
 
-    if (SUCCEEDED(hr = d2d_bitmap_create_shared(render_target->factory, render_target->device, iid, data, desc, &object)))
+    if (SUCCEEDED(hr = d2d_bitmap_create_shared(iface, render_target->device, iid, data, desc, &object)))
         *bitmap = &object->ID2D1Bitmap_iface;
 
     return hr;
@@ -426,12 +420,30 @@ static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateRadialGradientBrush
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateCompatibleRenderTarget(ID2D1RenderTarget *iface,
         const D2D1_SIZE_F *size, const D2D1_SIZE_U *pixel_size, const D2D1_PIXEL_FORMAT *format,
-        D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS options, ID2D1BitmapRenderTarget **render_target)
+        D2D1_COMPATIBLE_RENDER_TARGET_OPTIONS options, ID2D1BitmapRenderTarget **rt)
 {
-    FIXME("iface %p, size %p, pixel_size %p, format %p, options %#x, render_target %p stub!\n",
-            iface, size, pixel_size, format, options, render_target);
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    struct d2d_bitmap_render_target *object;
+    HRESULT hr;
 
-    return E_NOTIMPL;
+    TRACE("iface %p, size %p, pixel_size %p, format %p, options %#x, render_target %p.\n",
+            iface, size, pixel_size, format, options, rt);
+
+    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+        return E_OUTOFMEMORY;
+
+    if (FAILED(hr = d2d_bitmap_render_target_init(object, render_target, size, pixel_size,
+            format, options)))
+    {
+        WARN("Failed to initialize render target, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created render target %p.\n", object);
+    *rt = &object->ID2D1BitmapRenderTarget_iface;
+
+    return S_OK;
 }
 
 static HRESULT STDMETHODCALLTYPE d2d_d3d_render_target_CreateLayer(ID2D1RenderTarget *iface,
@@ -466,8 +478,21 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawLine(ID2D1RenderTarget *
 static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawRectangle(ID2D1RenderTarget *iface,
         const D2D1_RECT_F *rect, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
 {
-    FIXME("iface %p, rect %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    ID2D1RectangleGeometry *geometry;
+    HRESULT hr;
+
+    TRACE("iface %p, rect %p, brush %p, stroke_width %.8e, stroke_style %p.\n",
             iface, rect, brush, stroke_width, stroke_style);
+
+    if (FAILED(hr = ID2D1Factory_CreateRectangleGeometry(render_target->factory, rect, &geometry)))
+    {
+        ERR("Failed to create geometry, hr %#x.\n", hr);
+        return;
+    }
+
+    ID2D1RenderTarget_DrawGeometry(iface, (ID2D1Geometry *)geometry, brush, stroke_width, stroke_style);
+    ID2D1RectangleGeometry_Release(geometry);
 }
 
 static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRectangle(ID2D1RenderTarget *iface,
@@ -492,8 +517,21 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRectangle(ID2D1RenderTar
 static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawRoundedRectangle(ID2D1RenderTarget *iface,
         const D2D1_ROUNDED_RECT *rect, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
 {
-    FIXME("iface %p, rect %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    ID2D1RoundedRectangleGeometry *geometry;
+    HRESULT hr;
+
+    TRACE("iface %p, rect %p, brush %p, stroke_width %.8e, stroke_style %p.\n",
             iface, rect, brush, stroke_width, stroke_style);
+
+    if (FAILED(hr = ID2D1Factory_CreateRoundedRectangleGeometry(render_target->factory, rect, &geometry)))
+    {
+        ERR("Failed to create geometry, hr %#x.\n", hr);
+        return;
+    }
+
+    ID2D1RenderTarget_DrawGeometry(iface, (ID2D1Geometry *)geometry, brush, stroke_width, stroke_style);
+    ID2D1RoundedRectangleGeometry_Release(geometry);
 }
 
 static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRoundedRectangle(ID2D1RenderTarget *iface,
@@ -518,8 +556,21 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_FillRoundedRectangle(ID2D1Re
 static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawEllipse(ID2D1RenderTarget *iface,
         const D2D1_ELLIPSE *ellipse, ID2D1Brush *brush, float stroke_width, ID2D1StrokeStyle *stroke_style)
 {
-    FIXME("iface %p, ellipse %p, brush %p, stroke_width %.8e, stroke_style %p stub!\n",
+    struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
+    ID2D1EllipseGeometry *geometry;
+    HRESULT hr;
+
+    TRACE("iface %p, ellipse %p, brush %p, stroke_width %.8e, stroke_style %p.\n",
             iface, ellipse, brush, stroke_width, stroke_style);
+
+    if (FAILED(hr = ID2D1Factory_CreateEllipseGeometry(render_target->factory, ellipse, &geometry)))
+    {
+        ERR("Failed to create geometry, hr %#x.\n", hr);
+        return;
+    }
+
+    ID2D1RenderTarget_DrawGeometry(iface, (ID2D1Geometry *)geometry, brush, stroke_width, stroke_style);
+    ID2D1EllipseGeometry_Release(geometry);
 }
 
 static void STDMETHODCALLTYPE d2d_d3d_render_target_FillEllipse(ID2D1RenderTarget *iface,
@@ -563,8 +614,8 @@ static void d2d_rt_fill_geometry(struct d2d_d3d_render_target *render_target,
         float _12, _22, _32, pad1;
     } transform;
 
-    tmp_x =  (2.0f * render_target->dpi_x) / (96.0f * render_target->pixel_size.width);
-    tmp_y = -(2.0f * render_target->dpi_y) / (96.0f * render_target->pixel_size.height);
+    tmp_x =  (2.0f * render_target->desc.dpiX) / (96.0f * render_target->pixel_size.width);
+    tmp_y = -(2.0f * render_target->desc.dpiY) / (96.0f * render_target->pixel_size.height);
     w = render_target->drawing_state.transform;
     w._11 *= tmp_x;
     w._21 *= tmp_x;
@@ -787,7 +838,7 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawText(ID2D1RenderTarget *
                 layout_rect->right - layout_rect->left, layout_rect->bottom - layout_rect->top, &text_layout);
     else
         hr = IDWriteFactory_CreateGdiCompatibleTextLayout(dwrite_factory, string, string_len, text_format,
-                layout_rect->right - layout_rect->left, layout_rect->bottom - layout_rect->top, render_target->dpi_x / 96.0f,
+                layout_rect->right - layout_rect->left, layout_rect->bottom - layout_rect->top, render_target->desc.dpiX / 96.0f,
                 (DWRITE_MATRIX*)&render_target->drawing_state.transform, measuring_mode == DWRITE_MEASURING_MODE_GDI_NATURAL, &text_layout);
     IDWriteFactory_Release(dwrite_factory);
     if (FAILED(hr))
@@ -936,10 +987,10 @@ static void d2d_rt_draw_glyph_run_bitmap(struct d2d_d3d_render_target *render_ta
 
     bitmap_desc.pixelFormat.format = DXGI_FORMAT_A8_UNORM;
     bitmap_desc.pixelFormat.alphaMode = D2D1_ALPHA_MODE_PREMULTIPLIED;
-    bitmap_desc.dpiX = render_target->dpi_x;
+    bitmap_desc.dpiX = render_target->desc.dpiX;
     if (texture_type == DWRITE_TEXTURE_CLEARTYPE_3x1)
         bitmap_desc.dpiX *= 3.0f;
-    bitmap_desc.dpiY = render_target->dpi_y;
+    bitmap_desc.dpiY = render_target->desc.dpiY;
     if (FAILED(hr = d2d_d3d_render_target_CreateBitmap(&render_target->ID2D1RenderTarget_iface,
             bitmap_size, opacity_values, bitmap_size.width, &bitmap_desc, &opacity_bitmap)))
     {
@@ -1001,7 +1052,7 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_DrawGlyphRun(ID2D1RenderTarg
     if (render_target->drawing_state.textAntialiasMode != D2D1_TEXT_ANTIALIAS_MODE_DEFAULT)
         FIXME("Ignoring text antialiasing mode %#x.\n", render_target->drawing_state.textAntialiasMode);
 
-    ppd = max(render_target->dpi_x, render_target->dpi_y) / 96.0f;
+    ppd = max(render_target->desc.dpiX, render_target->desc.dpiY) / 96.0f;
     rendering_params = render_target->text_rendering_params ? render_target->text_rendering_params
             : render_target->default_text_rendering_params;
     if (FAILED(hr = IDWriteFontFace_GetRecommendedRenderingMode(glyph_run->fontFace, glyph_run->fontEmSize,
@@ -1184,8 +1235,8 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_PushAxisAlignedClip(ID2D1Ren
     if (antialias_mode != D2D1_ANTIALIAS_MODE_ALIASED)
         FIXME("Ignoring antialias_mode %#x.\n", antialias_mode);
 
-    x_scale = render_target->dpi_x / 96.0f;
-    y_scale = render_target->dpi_y / 96.0f;
+    x_scale = render_target->desc.dpiX / 96.0f;
+    y_scale = render_target->desc.dpiY / 96.0f;
     d2d_point_transform(&point, &render_target->drawing_state.transform,
             clip_rect->left * x_scale, clip_rect->top * y_scale);
     d2d_rect_set(&transformed_rect, point.x, point.y, point.x, point.y);
@@ -1247,7 +1298,7 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_Clear(ID2D1RenderTarget *ifa
 
     if (color)
         c = *color;
-    if (render_target->format.alphaMode == D2D1_ALPHA_MODE_IGNORE)
+    if (render_target->desc.pixelFormat.alphaMode == D2D1_ALPHA_MODE_IGNORE)
         c.a = 1.0f;
     c.r *= c.a;
     c.g *= c.a;
@@ -1300,7 +1351,7 @@ static D2D1_PIXEL_FORMAT * STDMETHODCALLTYPE d2d_d3d_render_target_GetPixelForma
 
     TRACE("iface %p, format %p.\n", iface, format);
 
-    *format = render_target->format;
+    *format = render_target->desc.pixelFormat;
     return format;
 }
 
@@ -1318,8 +1369,8 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_SetDpi(ID2D1RenderTarget *if
     else if (dpi_x <= 0.0f || dpi_y <= 0.0f)
         return;
 
-    render_target->dpi_x = dpi_x;
-    render_target->dpi_y = dpi_y;
+    render_target->desc.dpiX = dpi_x;
+    render_target->desc.dpiY = dpi_y;
 }
 
 static void STDMETHODCALLTYPE d2d_d3d_render_target_GetDpi(ID2D1RenderTarget *iface, float *dpi_x, float *dpi_y)
@@ -1328,8 +1379,8 @@ static void STDMETHODCALLTYPE d2d_d3d_render_target_GetDpi(ID2D1RenderTarget *if
 
     TRACE("iface %p, dpi_x %p, dpi_y %p.\n", iface, dpi_x, dpi_y);
 
-    *dpi_x = render_target->dpi_x;
-    *dpi_y = render_target->dpi_y;
+    *dpi_x = render_target->desc.dpiX;
+    *dpi_y = render_target->desc.dpiY;
 }
 
 static D2D1_SIZE_F * STDMETHODCALLTYPE d2d_d3d_render_target_GetSize(ID2D1RenderTarget *iface, D2D1_SIZE_F *size)
@@ -1338,8 +1389,8 @@ static D2D1_SIZE_F * STDMETHODCALLTYPE d2d_d3d_render_target_GetSize(ID2D1Render
 
     TRACE("iface %p, size %p.\n", iface, size);
 
-    size->width = render_target->pixel_size.width / (render_target->dpi_x / 96.0f);
-    size->height = render_target->pixel_size.height / (render_target->dpi_y / 96.0f);
+    size->width = render_target->pixel_size.width / (render_target->desc.dpiX / 96.0f);
+    size->height = render_target->pixel_size.height / (render_target->desc.dpiY / 96.0f);
     return size;
 }
 
@@ -1502,7 +1553,7 @@ static HRESULT STDMETHODCALLTYPE d2d_text_renderer_GetPixelsPerDip(IDWriteTextRe
 
     TRACE("iface %p, ctx %p, ppd %p.\n", iface, ctx, ppd);
 
-    *ppd = render_target->dpi_y / 96.0f;
+    *ppd = render_target->desc.dpiY / 96.0f;
 
     return S_OK;
 }
@@ -1514,6 +1565,7 @@ static HRESULT STDMETHODCALLTYPE d2d_text_renderer_DrawGlyphRun(IDWriteTextRende
     struct d2d_d3d_render_target *render_target = impl_from_IDWriteTextRenderer(iface);
     D2D1_POINT_2F baseline_origin = {baseline_origin_x, baseline_origin_y};
     struct d2d_draw_text_layout_ctx *context = ctx;
+    BOOL color_font = FALSE;
     ID2D1Brush *brush;
 
     TRACE("iface %p, ctx %p, baseline_origin_x %.8e, baseline_origin_y %.8e, "
@@ -1523,14 +1575,97 @@ static HRESULT STDMETHODCALLTYPE d2d_text_renderer_DrawGlyphRun(IDWriteTextRende
 
     if (desc)
         WARN("Ignoring glyph run description %p.\n", desc);
-    if (context->options & ~D2D1_DRAW_TEXT_OPTIONS_NO_SNAP)
+    if (context->options & ~(D2D1_DRAW_TEXT_OPTIONS_NO_SNAP | D2D1_DRAW_TEXT_OPTIONS_ENABLE_COLOR_FONT))
         FIXME("Ignoring options %#x.\n", context->options);
 
     brush = d2d_draw_get_text_brush(context, effect);
 
     TRACE("%s\n", debugstr_wn(desc->string, desc->stringLength));
-    ID2D1RenderTarget_DrawGlyphRun(&render_target->ID2D1RenderTarget_iface,
-            baseline_origin, glyph_run, brush, measuring_mode);
+
+    if (context->options & D2D1_DRAW_TEXT_OPTIONS_ENABLE_COLOR_FONT)
+    {
+        IDWriteFontFace2 *fontface;
+
+        if (SUCCEEDED(IDWriteFontFace_QueryInterface(glyph_run->fontFace,
+                &IID_IDWriteFontFace2, (void **)&fontface)))
+        {
+            color_font = IDWriteFontFace2_IsColorFont(fontface);
+            IDWriteFontFace2_Release(fontface);
+        }
+    }
+
+    if (color_font)
+    {
+        IDWriteColorGlyphRunEnumerator *layers;
+        IDWriteFactory2 *dwrite_factory;
+        HRESULT hr;
+
+        if (FAILED(hr = DWriteCreateFactory(DWRITE_FACTORY_TYPE_SHARED, &IID_IDWriteFactory2,
+                (IUnknown **)&dwrite_factory)))
+        {
+            ERR("Failed to create dwrite factory, hr %#x.\n", hr);
+            ID2D1Brush_Release(brush);
+            return hr;
+        }
+
+        hr = IDWriteFactory2_TranslateColorGlyphRun(dwrite_factory, baseline_origin_x, baseline_origin_y,
+                glyph_run, desc, measuring_mode, (DWRITE_MATRIX *)&render_target->drawing_state.transform, 0, &layers);
+        IDWriteFactory2_Release(dwrite_factory);
+        if (FAILED(hr))
+        {
+            ERR("Failed to create color glyph run enumerator, hr %#x.\n", hr);
+            ID2D1Brush_Release(brush);
+            return hr;
+        }
+
+        for (;;)
+        {
+            const DWRITE_COLOR_GLYPH_RUN *color_run;
+            ID2D1Brush *color_brush;
+            D2D1_POINT_2F origin;
+            BOOL has_run = FALSE;
+
+            if (FAILED(hr = IDWriteColorGlyphRunEnumerator_MoveNext(layers, &has_run)))
+            {
+                ERR("Failed to switch color glyph layer, hr %#x.\n", hr);
+                break;
+            }
+
+            if (!has_run)
+                break;
+
+            if (FAILED(hr = IDWriteColorGlyphRunEnumerator_GetCurrentRun(layers, &color_run)))
+            {
+                ERR("Failed to get current color run, hr %#x.\n", hr);
+                break;
+            }
+
+            if (color_run->paletteIndex == 0xffff)
+                color_brush = brush;
+            else
+            {
+                if (FAILED(hr = ID2D1RenderTarget_CreateSolidColorBrush(&render_target->ID2D1RenderTarget_iface,
+                        &color_run->runColor, NULL, (ID2D1SolidColorBrush **)&color_brush)))
+                {
+                    ERR("Failed to create solid color brush, hr %#x.\n", hr);
+                    break;
+                }
+            }
+
+            origin.x = color_run->baselineOriginX;
+            origin.y = color_run->baselineOriginY;
+            ID2D1RenderTarget_DrawGlyphRun(&render_target->ID2D1RenderTarget_iface,
+                    origin, &color_run->glyphRun, color_brush, measuring_mode);
+
+            if (color_brush != brush)
+                ID2D1Brush_Release(color_brush);
+        }
+
+        IDWriteColorGlyphRunEnumerator_Release(layers);
+    }
+    else
+        ID2D1RenderTarget_DrawGlyphRun(&render_target->ID2D1RenderTarget_iface,
+                baseline_origin, glyph_run, brush, measuring_mode);
 
     ID2D1Brush_Release(brush);
 
@@ -1551,7 +1686,7 @@ static HRESULT STDMETHODCALLTYPE d2d_text_renderer_DrawUnderline(IDWriteTextRend
             iface, ctx, baseline_origin_x, baseline_origin_y, underline, effect);
 
     /* minimal thickness in DIPs that will result in at least 1 pixel thick line */
-    min_thickness = 96.0f / (render_target->dpi_y * sqrtf(m->_21 * m->_21 + m->_22 * m->_22));
+    min_thickness = 96.0f / (render_target->desc.dpiY * sqrtf(m->_21 * m->_21 + m->_22 * m->_22));
 
     rect.left   = baseline_origin_x;
     rect.top    = baseline_origin_y + underline->offset;
@@ -2128,7 +2263,7 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         goto err;
     }
 
-    render_target->format = desc->pixelFormat;
+    render_target->desc.pixelFormat = desc->pixelFormat;
     render_target->pixel_size.width = surface_desc.Width;
     render_target->pixel_size.height = surface_desc.Height;
     render_target->drawing_state.transform = identity;
@@ -2140,8 +2275,8 @@ HRESULT d2d_d3d_render_target_init(struct d2d_d3d_render_target *render_target,
         goto err;
     }
 
-    render_target->dpi_x = dpi_x;
-    render_target->dpi_y = dpi_y;
+    render_target->desc.dpiX = dpi_x;
+    render_target->desc.dpiY = dpi_y;
 
     return S_OK;
 
@@ -2181,7 +2316,7 @@ err:
     return hr;
 }
 
-HRESULT d2d_d3d_render_target_update_surface(ID2D1RenderTarget *iface, IDXGISurface1 *surface)
+HRESULT d2d_d3d_render_target_create_rtv(ID2D1RenderTarget *iface, IDXGISurface1 *surface)
 {
     struct d2d_d3d_render_target *render_target = impl_from_ID2D1RenderTarget(iface);
     DXGI_SURFACE_DESC surface_desc;
@@ -2189,6 +2324,13 @@ HRESULT d2d_d3d_render_target_update_surface(ID2D1RenderTarget *iface, IDXGISurf
     ID3D10Resource *resource;
     HRESULT hr;
 
+    if (!surface)
+    {
+        ID3D10RenderTargetView_Release(render_target->view);
+        render_target->view = NULL;
+        return S_OK;
+    }
+
     if (FAILED(hr = IDXGISurface1_GetDesc(surface, &surface_desc)))
     {
         WARN("Failed to get surface desc, hr %#x.\n", hr);
