@@ -92,6 +92,8 @@ static CRITICAL_SECTION_DEBUG critsect_debug =
 };
 static CRITICAL_SECTION wgl_section = { &critsect_debug, -1, 0, 0, 0, 0 };
 
+static const MAT2 identity = { {0,1},{0,0},{0,0},{0,1} };
+
 static inline struct opengl_funcs *get_dc_funcs( HDC hdc )
 {
     struct opengl_funcs *funcs = __wine_get_wgl_driver( hdc, WINE_WGL_DRIVER_VERSION );
@@ -827,7 +829,7 @@ static BOOL is_extension_supported(const char* extension)
 
             /* Compare the major/minor version numbers of the native OpenGL library and what is required by the function.
              * The gl_version string is guaranteed to have at least a major/minor and sometimes it has a release number as well. */
-            if( (gl_version[0] >= version[0]) || ((gl_version[0] == version[0]) && (gl_version[2] >= version[2])) ) {
+            if( (gl_version[0] > version[0]) || ((gl_version[0] == version[0]) && (gl_version[2] >= version[2])) ) {
                 return TRUE;
             }
             WARN("The function requires OpenGL version '%c.%c' while your drivers only provide '%c.%c'\n", version[0], version[2], gl_version[0], gl_version[2]);
@@ -876,6 +878,20 @@ PROC WINAPI wglGetProcAddress( LPCSTR name )
 
         if (!is_extension_supported(ext_ret->extension))
         {
+            unsigned int i;
+            static const struct { const char *name, *alt; } alternatives[] =
+            {
+                { "glCopyTexSubImage3DEXT", "glCopyTexSubImage3D" },     /* needed by RuneScape */
+                { "glVertexAttribDivisor", "glVertexAttribDivisorARB"},  /* needed by Caffeine */
+            };
+
+            for (i = 0; i < sizeof(alternatives)/sizeof(alternatives[0]); i++)
+            {
+                if (strcmp( name, alternatives[i].name )) continue;
+                WARN("Extension %s required for %s not supported, trying %s\n",
+                    ext_ret->extension, name, alternatives[i].alt );
+                return wglGetProcAddress( alternatives[i].alt );
+            }
             WARN("Extension %s required for %s not supported\n", ext_ret->extension, name);
             return NULL;
         }
@@ -1057,7 +1073,7 @@ BOOL WINAPI wglGetPixelFormatAttribfvARB( HDC hdc, int format, int layer, UINT c
  */
 HPBUFFERARB WINAPI wglCreatePbufferARB( HDC hdc, int format, int width, int height, const int *attribs )
 {
-    HPBUFFERARB ret = 0;
+    HPBUFFERARB ret;
     struct wgl_pbuffer *pbuffer;
     struct opengl_funcs *funcs = get_dc_funcs( hdc );
 
@@ -1196,6 +1212,58 @@ BOOL WINAPI wglSetPixelFormatWINE( HDC hdc, int format )
     return funcs->ext.p_wglSetPixelFormatWINE( hdc, format );
 }
 
+/***********************************************************************
+ *              wglQueryCurrentRendererIntegerWINE
+ *
+ * Provided by the WGL_WINE_query_renderer extension.
+ */
+BOOL WINAPI wglQueryCurrentRendererIntegerWINE( GLenum attribute, GLuint *value )
+{
+    const struct opengl_funcs *funcs = NtCurrentTeb()->glTable;
+
+    if (!funcs->ext.p_wglQueryCurrentRendererIntegerWINE) return FALSE;
+    return funcs->ext.p_wglQueryCurrentRendererIntegerWINE( attribute, value );
+}
+
+/***********************************************************************
+ *              wglQueryCurrentRendererStringWINE
+ *
+ * Provided by the WGL_WINE_query_renderer extension.
+ */
+const GLchar * WINAPI wglQueryCurrentRendererStringWINE( GLenum attribute )
+{
+    const struct opengl_funcs *funcs = NtCurrentTeb()->glTable;
+
+    if (!funcs->ext.p_wglQueryCurrentRendererStringWINE) return NULL;
+    return funcs->ext.p_wglQueryCurrentRendererStringWINE( attribute );
+}
+
+/***********************************************************************
+ *              wglQueryRendererIntegerWINE
+ *
+ * Provided by the WGL_WINE_query_renderer extension.
+ */
+BOOL WINAPI wglQueryRendererIntegerWINE( HDC dc, GLint renderer, GLenum attribute, GLuint *value )
+{
+    const struct opengl_funcs *funcs = get_dc_funcs( dc );
+
+    if (!funcs || !funcs->ext.p_wglQueryRendererIntegerWINE) return FALSE;
+    return funcs->ext.p_wglQueryRendererIntegerWINE( dc, renderer, attribute, value );
+}
+
+/***********************************************************************
+ *              wglQueryRendererStringWINE
+ *
+ * Provided by the WGL_WINE_query_renderer extension.
+ */
+const GLchar * WINAPI wglQueryRendererStringWINE( HDC dc, GLint renderer, GLenum attribute )
+{
+    const struct opengl_funcs *funcs = get_dc_funcs( dc );
+
+    if (!funcs || !funcs->ext.p_wglQueryRendererStringWINE) return NULL;
+    return funcs->ext.p_wglQueryRendererStringWINE( dc, renderer, attribute );
+}
+
 /***********************************************************************
  *		wglUseFontBitmaps_common
  */
@@ -1212,7 +1280,6 @@ static BOOL wglUseFontBitmaps_common( HDC hdc, DWORD first, DWORD count, DWORD l
      funcs->gl.p_glPixelStorei(GL_UNPACK_ALIGNMENT, 4);
 
      for (glyph = first; glyph < first + count; glyph++) {
-         static const MAT2 identity = { {0,1},{0,0},{0,0},{0,1} };
          unsigned int needed_size, height, width, width_int;
 
          if (unicode)
@@ -1479,7 +1546,6 @@ static BOOL wglUseFontOutlines_common(HDC hdc,
 {
     const struct opengl_funcs *funcs = NtCurrentTeb()->glTable;
     UINT glyph;
-    const MAT2 identity = {{0,1},{0,0},{0,0},{0,1}};
     GLUtesselator *tess = NULL;
     LOGFONTW lf;
     HFONT old_font, unscaled_font;
@@ -1749,12 +1815,15 @@ GLint WINAPI glDebugEntry( GLint unknown1, GLint unknown2 )
 
 static GLubyte *filter_extensions_list(const char *extensions, const char *disabled)
 {
-    char *p, *str = NULL;
+    char *p, *str;
     const char *end;
 
     p = str = HeapAlloc(GetProcessHeap(), 0, strlen(extensions) + 2);
     if (!str)
         return NULL;
+
+    TRACE( "GL_EXTENSIONS:\n" );
+
     for (;;)
     {
         while (*extensions == ' ')
@@ -1785,7 +1854,7 @@ static GLuint *filter_extensions_index(const char *disabled)
 {
     const struct opengl_funcs *funcs = NtCurrentTeb()->glTable;
     const char *ext, *end, *gl_ext;
-    GLuint *disabled_exts = NULL, *new_disabled_exts;
+    GLuint *disabled_exts, *new_disabled_exts;
     unsigned int i = 0, j, disabled_size;
     GLint extensions_count;
 
@@ -1803,6 +1872,9 @@ static GLuint *filter_extensions_index(const char *disabled)
     disabled_exts = HeapAlloc(GetProcessHeap(), 0, disabled_size * sizeof(*disabled_exts));
     if (!disabled_exts)
         return NULL;
+
+    TRACE( "GL_EXTENSIONS:\n" );
+
     for (j = 0; j < extensions_count; ++j)
     {
         gl_ext = (const char *)funcs->ext.p_glGetStringi(GL_EXTENSIONS, j);
@@ -1849,8 +1921,6 @@ static BOOL filter_extensions(const char *extensions, GLubyte **exts_list, GLuin
 {
     static const char *disabled;
 
-    TRACE( "GL_EXTENSIONS:\n" );
-
     if (!disabled)
     {
         HKEY hkey;
