@@ -135,7 +135,7 @@ static void draw_primitive_arrays(struct wined3d_context *context, const struct
             element = &si->elements[element_idx];
             ptr = element->data.addr + element->stride * i;
             if (element->data.buffer_object)
-                ptr += (ULONG_PTR)buffer_get_sysmem(state->streams[element->stream_idx].buffer, context);
+                ptr += (ULONG_PTR)wined3d_buffer_load_sysmem(state->streams[element->stream_idx].buffer, context);
             ops->generic[element->format->emit_idx](element_idx, ptr);
         }
 
@@ -196,12 +196,10 @@ static void draw_primitive_immediate_mode(struct wined3d_context *context, const
     if (instance_count)
         FIXME("Instancing not implemented.\n");
 
-    /* Immediate mode drawing can't make use of indices in a vbo - get the
-     * data from the index buffer. If the index buffer has no vbo (not
-     * supported or other reason), or with user pointer drawing idx_data
-     * will be non-NULL. */
-    if (idx_size && !idx_data)
-        idx_data = buffer_get_sysmem(state->index_buffer, context);
+    /* Immediate mode drawing can't make use of indices in a VBO - get the
+     * data from the index buffer. */
+    if (idx_size)
+        idx_data = wined3d_buffer_load_sysmem(state->index_buffer, context) + state->index_offset;
 
     ops = &d3d_info->ffp_attrib_ops;
 
@@ -391,18 +389,19 @@ static void remove_vbos(struct wined3d_context *context,
 {
     unsigned int i;
 
-    for (i = 0; i < (sizeof(s->elements) / sizeof(*s->elements)); ++i)
+    for (i = 0; i < ARRAY_SIZE(s->elements); ++i)
     {
         struct wined3d_stream_info_element *e;
 
-        if (!(s->use_map & (1u << i))) continue;
+        if (!(s->use_map & (1u << i)))
+            continue;
 
         e = &s->elements[i];
         if (e->data.buffer_object)
         {
             struct wined3d_buffer *vb = state->streams[e->stream_idx].buffer;
             e->data.buffer_object = 0;
-            e->data.addr = (BYTE *)((ULONG_PTR)e->data.addr + (ULONG_PTR)buffer_get_sysmem(vb, context));
+            e->data.addr += (ULONG_PTR)wined3d_buffer_load_sysmem(vb, context);
         }
     }
 }
