@@ -33,14 +33,6 @@
 
 DEFINE_GUID(IID_IXmlReaderInput, 0x0b3ccc9b, 0x9214, 0x428b, 0xa2, 0xae, 0xef, 0x3a, 0xa8, 0x71, 0xaf, 0xda);
 
-static HRESULT (WINAPI *pCreateXmlReader)(REFIID riid, void **ppvObject, IMalloc *pMalloc);
-static HRESULT (WINAPI *pCreateXmlReaderInputWithEncodingName)(IUnknown *stream,
-                                                        IMalloc *pMalloc,
-                                                        LPCWSTR encoding,
-                                                        BOOL hint,
-                                                        LPCWSTR base_uri,
-                                                        IXmlReaderInput **ppInput);
-
 static WCHAR *a2w(const char *str)
 {
     int len = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
@@ -367,25 +359,6 @@ static const ISequentialStreamVtbl teststreamvtbl =
     teststream_Write
 };
 
-static BOOL init_pointers(void)
-{
-    /* don't free module here, it's to be unloaded on exit */
-    HMODULE mod = LoadLibraryA("xmllite.dll");
-
-    if (!mod)
-    {
-        win_skip("xmllite library not available\n");
-        return FALSE;
-    }
-
-#define MAKEFUNC(f) if (!(p##f = (void*)GetProcAddress(mod, #f))) return FALSE;
-    MAKEFUNC(CreateXmlReader);
-    MAKEFUNC(CreateXmlReaderInputWithEncodingName);
-#undef MAKEFUNC
-
-    return TRUE;
-}
-
 static HRESULT WINAPI resolver_QI(IXmlResolver *iface, REFIID riid, void **obj)
 {
     ok(0, "unexpected call, riid %s\n", wine_dbgstr_guid(riid));
@@ -440,11 +413,11 @@ static void test_reader_create(void)
     /* crashes native */
     if (0)
     {
-        pCreateXmlReader(&IID_IXmlReader, NULL, NULL);
-        pCreateXmlReader(NULL, (void**)&reader, NULL);
+        CreateXmlReader(&IID_IXmlReader, NULL, NULL);
+        CreateXmlReader(NULL, (void**)&reader, NULL);
     }
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     test_read_state(reader, XmlReadState_Closed, -1, FALSE);
@@ -517,9 +490,9 @@ static void test_readerinput(void)
     HRESULT hr;
     LONG ref;
 
-    hr = pCreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, NULL);
+    hr = CreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, NULL);
     ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
-    hr = pCreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, &reader_input);
+    hr = CreateXmlReaderInputWithEncodingName(NULL, NULL, NULL, FALSE, NULL, &reader_input);
     ok(hr == E_INVALIDARG, "Expected E_INVALIDARG, got %08x\n", hr);
 
     hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
@@ -528,7 +501,7 @@ static void test_readerinput(void)
     ref = IStream_AddRef(stream);
     ok(ref == 2, "Expected 2, got %d\n", ref);
     IStream_Release(stream);
-    hr = pCreateXmlReaderInputWithEncodingName((IUnknown*)stream, NULL, NULL, FALSE, NULL, &reader_input);
+    hr = CreateXmlReaderInputWithEncodingName((IUnknown*)stream, NULL, NULL, FALSE, NULL, &reader_input);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IUnknown_QueryInterface(reader_input, &IID_IStream, (void**)&stream2);
@@ -543,7 +516,7 @@ static void test_readerinput(void)
     IStream_Release(stream);
 
     /* try ::SetInput() with valid IXmlReaderInput */
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     ref = IUnknown_AddRef(reader_input);
@@ -609,7 +582,7 @@ static void test_readerinput(void)
     ref = IUnknown_AddRef(input);
     ok(ref == 2, "Expected 2, got %d\n", ref);
     IUnknown_Release(input);
-    hr = pCreateXmlReaderInputWithEncodingName(input, NULL, NULL, FALSE, NULL, &reader_input);
+    hr = CreateXmlReaderInputWithEncodingName(input, NULL, NULL, FALSE, NULL, &reader_input);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
     ok_iids(&input_iids, empty_seq, NULL, FALSE);
     /* IXmlReaderInput stores stream interface as IUnknown */
@@ -617,7 +590,7 @@ static void test_readerinput(void)
     ok(ref == 3, "Expected 3, got %d\n", ref);
     IUnknown_Release(input);
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     input_iids.count = 0;
@@ -648,7 +621,7 @@ static void test_readerinput(void)
     ok_iids(&input_iids, setinput_readerinput, NULL, FALSE);
 
     /* another reader */
-    hr = pCreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader2, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader2, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* resolving from IXmlReaderInput to IStream/ISequentialStream is done at
@@ -671,7 +644,7 @@ static void test_reader_state(void)
     XmlNodeType nodetype;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* invalid arguments */
@@ -711,7 +684,7 @@ static void test_read_xmldeclaration(void)
     BOOL ret;
     const WCHAR *val;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (LPVOID*)&reader, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* position methods with Null args */
@@ -932,7 +905,7 @@ static void test_read_comment(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1014,7 +987,7 @@ static void test_read_pi(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1130,7 +1103,7 @@ static void test_read_full(void)
     HRESULT hr;
     int i;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     stream = create_stream_on_data(test->xml, strlen(test->xml)+1);
@@ -1179,7 +1152,7 @@ static void test_read_dtd(void)
     UINT len, count;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     hr = IXmlReader_SetProperty(reader, XmlReaderProperty_DtdProcessing, DtdProcessing_Parse);
@@ -1272,7 +1245,7 @@ static void test_read_element(void)
     UINT depth;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1403,7 +1376,7 @@ static void test_read_pending(void)
     HRESULT hr;
     int c;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got 0x%08x\n", hr);
 
     hr = IXmlReader_SetInput(reader, (IUnknown*)&teststream);
@@ -1441,7 +1414,7 @@ static void test_readvaluechunk(void)
     HRESULT hr;
     UINT c;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     stream = create_stream_on_data(testA, sizeof(testA));
@@ -1496,7 +1469,7 @@ static void test_read_cdata(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1589,7 +1562,7 @@ static void test_read_text(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1687,7 +1660,7 @@ static void test_isemptyelement(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1745,7 +1718,7 @@ static void test_read_attribute(void)
     IXmlReader *reader;
     HRESULT hr;
 
-    hr = pCreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
     ok(hr == S_OK, "S_OK, got %08x\n", hr);
 
     while (test->xml)
@@ -1812,11 +1785,25 @@ static void test_read_attribute(void)
     IXmlReader_Release(reader);
 }
 
+static void test_reader_properties(void)
+{
+    IXmlReader *reader;
+    HRESULT hr;
+
+    hr = CreateXmlReader(&IID_IXmlReader, (void**)&reader, NULL);
+    ok(hr == S_OK, "S_OK, got %08x\n", hr);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_MultiLanguage, 0);
+    ok(hr == S_OK, "SetProperty failed: %08x\n", hr);
+
+    hr = IXmlReader_SetProperty(reader, XmlReaderProperty_MaxElementDepth, 0);
+    ok(hr == S_OK, "SetProperty failed: %08x\n", hr);
+
+    IXmlReader_Release(reader);
+}
+
 START_TEST(reader)
 {
-    if (!init_pointers())
-       return;
-
     test_reader_create();
     test_readerinput();
     test_reader_state();
@@ -1832,4 +1819,5 @@ START_TEST(reader)
     test_read_pending();
     test_readvaluechunk();
     test_read_xmldeclaration();
+    test_reader_properties();
 }
